#define ora_getptr_Lda_DefPtr ora_getptr_generic
#define ora_getptr_OCIAQDeqOptionsPtr ora_getptr_generic
#define ora_getptr_OCIAQEnqOptionsPtr ora_getptr_generic
#define ora_getptr_OCIAQMsgPropertiesPtr ora_getptr_generic
#define ora_getptr_OCIBindPtr ora_getptr_generic
#define ora_getptr_OCICallbackDefine ora_getptr_generic
#define ora_getptr_OCICallbackInBind ora_getptr_generic
#define ora_getptr_OCICallbackOutBind ora_getptr_generic
#define ora_getptr_OCICollPtr ora_getptr_generic
#define ora_getptr_OCIComplexObjectPtr ora_getptr_generic
#define ora_getptr_OCIDatePtr ora_getptr_generic
#define ora_getptr_OCIDateTimePtr ora_getptr_generic
#define ora_getptr_OCIDefinePtr ora_getptr_generic
#define ora_getptr_OCIDescribePtr ora_getptr_generic
#define ora_getptr_OCIDirPathColArrayPtr ora_getptr_generic
#define ora_getptr_OCIDirPathCtxPtr ora_getptr_generic
#define ora_getptr_OCIDirPathStreamPtr ora_getptr_generic
#define ora_getptr_OCIDurationPtr ora_getptr_generic
#define ora_getptr_OCIEnvPtr ora_getptr_generic
#define ora_getptr_OCIErrorPtr ora_getptr_generic
#define ora_getptr_OCIFileObjectPtr ora_getptr_generic
#define ora_getptr_OCIIndPtr ora_getptr_generic
#define ora_getptr_OCIIntervalPtr ora_getptr_generic
#define ora_getptr_OCIIterPtr ora_getptr_generic
#define ora_getptr_OCILobLocatorPtr ora_getptr_generic
#define ora_getptr_OCILockOpt ora_getptr_generic
#define ora_getptr_OCIMsgPtr ora_getptr_generic
#define ora_getptr_OCINumberPtr ora_getptr_generic
#define ora_getptr_OCIObjectEvent ora_getptr_generic
#define ora_getptr_OCIObjectProperty ora_getptr_generic
#define ora_getptr_OCIPicklerFdoPtr ora_getptr_generic
#define ora_getptr_OCIPicklerImagePtr ora_getptr_generic
#define ora_getptr_OCIPicklerTdsCtxPtr ora_getptr_generic
#define ora_getptr_OCIPicklerTdsElementPtr ora_getptr_generic
#define ora_getptr_OCIPicklerTdsPtr ora_getptr_generic
#define ora_getptr_OCIPinOpt ora_getptr_generic
#define ora_getptr_OCIRawPtr ora_getptr_generic
#define ora_getptr_OCIRefPtr ora_getptr_generic
#define ora_getptr_OCIRefreshOpt ora_getptr_generic
#define ora_getptr_OCIResultPtr ora_getptr_generic
#define ora_getptr_OCISecurityPtr ora_getptr_generic
#define ora_getptr_OCIServerPtr ora_getptr_generic
#define ora_getptr_OCISessionPtr ora_getptr_generic
#define ora_getptr_OCISnapshotPtr ora_getptr_generic
#define ora_getptr_OCIStmtPtr ora_getptr_generic
#define ora_getptr_OCIStringPtr ora_getptr_generic
#define ora_getptr_OCISubscriptionPtr ora_getptr_generic
#define ora_getptr_OCISvcCtxPtr ora_getptr_generic
#define ora_getptr_OCITablePtr ora_getptr_generic
#define ora_getptr_OCIThreadHandlePtr ora_getptr_generic
#define ora_getptr_OCIThreadIdPtr ora_getptr_generic
#define ora_getptr_OCIThreadKeyDestFunc ora_getptr_generic
#define ora_getptr_OCIThreadKeyPtr ora_getptr_generic
#define ora_getptr_OCIThreadMutexPtr ora_getptr_generic
#define ora_getptr_OCITransPtr ora_getptr_generic
#define ora_getptr_OCITypeCodePtr ora_getptr_generic
#define ora_getptr_OCITypeElemPtr ora_getptr_generic
#define ora_getptr_OCITypeGetOpt ora_getptr_generic
#define ora_getptr_OCITypeIterPtr ora_getptr_generic
#define ora_getptr_OCITypeMethodPtr ora_getptr_generic
#define ora_getptr_OCITypePtr ora_getptr_generic
#define ora_getptr_OCIUcbPtr ora_getptr_generic
#define ora_getptr_OCIUserCallback ora_getptr_generic
#define ora_getptr_OCIUserCallbackPtr ora_getptr_generic
#define ora_getptr_OCIWcharPtr ora_getptr_generic
#define ora_getptr_nzttBufferBlockPtr ora_getptr_generic
#define ora_getptr_nzttCipherType ora_getptr_generic
#define ora_getptr_nzttIdentType ora_getptr_generic
#define ora_getptr_nzttIdentityDescPtr ora_getptr_generic
#define ora_getptr_nzttIdentityPtr ora_getptr_generic
#define ora_getptr_nzttPersonaDescPtr ora_getptr_generic
#define ora_getptr_nzttPersonaPtr ora_getptr_generic
#define ora_getptr_nzttProtInfoPtr ora_getptr_generic
#define ora_getptr_nzttWalletPtr ora_getptr_generic
#define ora_getptr_nzttcef ora_getptr_generic
#define ora_getptr_nzttces ora_getptr_generic
#define ora_getptr_nztttdufmt ora_getptr_generic
#define ora_getptr_nztttdufmtPtr ora_getptr_generic
#define ora_getptr_sbig_oraPtr ora_getptr_generic
#define ora_getptr_ub1PtrPtr ora_getptr_generic
#define ora_getptr_ub4Ptr ora_getptr_generic
#define ora_getptr_voidPtr ora_getptr_generic
#define ora_getptr_voidPtrPtr ora_getptr_generic
