
/*
	unspeakable include file indirection hack (include "XSUB.h" => <XSUB.h>)
	just to get some macros defined
	without having to edit the generated OCI.xs file :)
*/
#include <XSUB.h>

#include "utility.h"
